#version 330

#define M_PI 3.1415926535897932384626433832795

uniform sampler2D colorTexture;
uniform sampler2D fireTexture;
uniform sampler2D noiseTexture;

uniform  vec4 fogColor; // Fog color
uniform float fogStart; // This is only for linear fog
uniform float fogEnd; // This is only for linear fog
uniform float fogDensity; // For exp and exp2 equation 
uniform int fogEquation; // 0 = linear, 1 = exp, 2 = exp2

uniform float timePassed;

in vec2 fUVs;
in vec3 N; //normal
in vec3 V; //view
in vec3 P; //position


in vec3 vertViewDist; //distance view to vertex

out vec4 fragColor;

float getFogFactor(int fogEquation, float fogStart, float fogEnd, float fogDensity, float fogCoord)
{
   float fResult = 0.0;
   if(fogEquation == 0)
      fResult = (fogEnd-fogCoord)/(fogEnd - fogStart);
   else if(fogEquation == 1)
      fResult = exp(-fogDensity*fogCoord);
   else if(fogEquation == 2)
      fResult = exp(-pow(fogDensity*fogCoord, 2.0));
      
   fResult = 1.0-clamp(fResult, 0.0, 1.0);

   return fResult;
}
 
void main() {
	vec3 C = texture(colorTexture, fUVs).rgb * 0.75;

	vec2 uvOffset = vec2(mod(timePassed*-0.25f, 1.0f), mod(timePassed*0.5f, 1.0f)); 
	vec3 FireC = texture(fireTexture, fUVs+uvOffset).rgb;

	vec3 FireC2 = texture(fireTexture, fUVs).rgb;
	float a2 = texture(noiseTexture, fUVs).x * ((sin((timePassed+0.5f)* 2.5f * M_PI)*0.25f)+0.25f);

	uvOffset = vec2(mod(timePassed*0.5f, 1.0f), mod(timePassed*-0.5f, 1.0f)); 
	float a = texture(noiseTexture, fUVs+uvOffset).x * ((sin(timePassed * M_PI)*0.20f)+0.6f);

	fragColor= vec4(C + (FireC * a) + (FireC2 * a2), 1.0);
	 
	float fogCoord = length(vertViewDist); 
	float fogfactor = getFogFactor(fogEquation, fogStart, fogEnd, fogDensity, fogCoord);
	fragColor = mix(fragColor, fogColor, fogfactor);
}