#version 330

uniform struct Matrices
{
   mat4 projMat;
   mat4 viewMat;
} matrices;

uniform vec3 quad1, quad2;

uniform int animRows;
uniform int animCols;
uniform int animFrames;
uniform float animSpeed;
uniform float blendDelay; //Blend in delay for smooth creation

layout(points) in;
layout(triangle_strip) out;
layout(max_vertices = 4) out;

in vec3 color_Pass[];
in float lifetime_Pass[];
in float size_Pass[];
in int type_Pass[];
in float timeleft_Pass[];

//smooth
out vec2 texCoord;
flat out vec4 colorPart;

void main()
{
  //if(type_Pass[0] != 0)
  //{
    vec3 pos_Old = gl_in[0].gl_Position.xyz;
    float size = size_Pass[0];
    mat4 VPmat = matrices.projMat*matrices.viewMat;
    float f = 0.0f;
	float BID = 0.0f;

	//extra: blend-in phase at particle spawn
	if (blendDelay > 0.0f) {
		BID = blendDelay; //blend in delay
		f = min(1.0f, 1.0f-((timeleft_Pass[0]-(1.0f-BID))*(1.0f/BID))) * (1.0f / (1.0f- BID)); 
		colorPart = vec4(color_Pass[0], timeleft_Pass[0] * f);
	} else {
		colorPart = vec4(color_Pass[0], timeleft_Pass[0]);
	}

	//uvCoords

	float progress = ((lifetime_Pass[0] - timeleft_Pass[0]) / lifetime_Pass[0]) * animSpeed;
	int frame = int(mod(int(progress*animFrames), animFrames));
	
	int col = int(mod(frame, animCols));
	int row = int(frame / animCols);
			
	float u = 1.0f / animCols;
	float v = 1.0f / animRows;

	float u0 = col * u;
	float u1 = (col+1) * u;
	float v0 = row * v;
	float v1 = (row+1) * v;

 /* //DEBUG CODE
    colorPart = vec4(color_Pass[0], timeleft_Pass[0]);
	float u0 = 0.0f;
	float u1 = 1.0f;
	float v0 = 0.0f;
	float v1 = 1.0f;
 */
	   
	//create particle quad vertices
    vec3 pos = pos_Old + (-quad1 - quad2) * size;
    texCoord = vec2(u0, v0);
    gl_Position = VPmat * vec4(pos, 1.0);
	EmitVertex();
    
    pos = pos_Old + (-quad1 + quad2 ) * size;
    texCoord = vec2(u0, v1);
    gl_Position = VPmat * vec4(pos, 1.0);
    EmitVertex();
    
    pos = pos_Old+(quad1 - quad2) * size;
    texCoord = vec2(u1, v0);
    gl_Position = VPmat * vec4(pos, 1.0);
    EmitVertex();
    
    pos = pos_Old + (quad1 + quad2) * size;
    texCoord = vec2(u1, v1);
    gl_Position = VPmat * vec4(pos, 1.0);
    EmitVertex();
	
/*
	//Debug code:
	vec3 pos = pos_Old;

	texCoord = vec2(u0, v0);
	gl_Position = VPmat * vec4((pos + vec3(texCoord, 0.0)), 1.0);
    EmitVertex();
    texCoord = vec2(u0, v1);
	gl_Position = VPmat * vec4((pos + vec3(texCoord, 0.0)), 1.0);
    EmitVertex();
	texCoord = vec2(u1, v0);
	gl_Position = VPmat * vec4((pos + vec3(texCoord, 0.0)), 1.0);
    EmitVertex();
	texCoord = vec2(u1, v1);
	gl_Position = VPmat * vec4((pos + vec3(texCoord, 0.0)), 1.0);
    EmitVertex();
*/
      
    EndPrimitive();
  //}
}