#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 60) out;

// in from vertex shader

in vec3 position_Pass[];
in vec3 velocity_Pass[];
in vec3 color_Pass[];
in float lifetime_Pass[];
in float timeleft_Pass[];
in float size_Pass[];
in int type_Pass[];

// out to fragment shader

out vec3 position_Out;
out vec3 velocity_Out;
out vec3 color_Out;
out float lifetime_Out;
out float timeleft_Out;
out float size_Out;
out int type_Out;

uniform vec3 genPosition1; // Particle generation positions, randomly create particles ..
uniform vec3 genPosition2; // .. on a line between this positions
uniform vec3 genGravity; // Gravity Vector
uniform vec3 genVelocity_Min; // New Particle Velocity - [min - min+range]
uniform vec3 genVelocity_Range;
uniform vec3 genColor;
uniform float genSize_Min, genSize_Range; 
uniform float genLifetime_Min, genLifetime_Range; // new particle lifetime - [min - min+range]
uniform float timePassed; // Time that passed since last frame
uniform int particleCount; // Particles to be created in next frame

//---
//Random number generation
// - from http://www.mbsoftworks.sk/index.php?page=tutorials&series=1&tutorial=26
// - origininal source unknown
uniform vec3 randomSeed; // Seed number for our random number function
vec3 localSeed;
float randZeroOne()
{
    uint n = floatBitsToUint(localSeed.y * 214013.0 + localSeed.x * 2531011.0 + localSeed.z * 141251.0);
    n = n * (n * n * 15731u + 789221u);
    n = (n >> 9u) | 0x3F800000u; 
    float fRes =  2.0 - uintBitsToFloat(n);
    localSeed = vec3(localSeed.x + 147158.0 * fRes, localSeed.y*fRes  + 415161.0 * fRes, localSeed.z + 324154.0*fRes);
    return fRes;
}
//----


void main()
{
  localSeed = randomSeed;
  
  position_Out = position_Pass[0];
  velocity_Out = velocity_Pass[0];
  if (type_Pass[0] != 0) {
	position_Out += velocity_Out * timePassed;
	velocity_Out += genGravity * timePassed;
	type_Out = 1;
  }

  color_Out = color_Pass[0];
  timeleft_Out = timeleft_Pass[0]-timePassed;
  lifetime_Out = lifetime_Pass[0];
  size_Out = size_Pass[0];
  //type_Out = type_Pass[0];

  vec3 posOffset = vec3(0.0, 0.0, 0.0);
    
  if(type_Pass[0] == 0)
  {
	position_Out = vec3(0.0, 0.0, 0.0);
	velocity_Out = vec3(0.0, 0.0, 0.0);
	color_Out = vec3(0.0, 0.0, 0.0);
	timeleft_Out = 1.0;
	lifetime_Out = 1.0;
	size_Out = 1.0;
	type_Out = 0;

    EmitVertex();
    EndPrimitive();
    
	posOffset = genPosition2 - genPosition1;

    for(int i = 0; i < particleCount; i++)
    {
      position_Out = genPosition1 + vec3(posOffset.x * randZeroOne(), posOffset.y * randZeroOne(), posOffset.z * randZeroOne());
      velocity_Out = genVelocity_Min + vec3(genVelocity_Range.x * randZeroOne(), genVelocity_Range.y * randZeroOne(), genVelocity_Range.z * randZeroOne());
      
	  //position_Out = vec3(-2.5f, 20.0f, 0.0f); //genPosition1;
	  //velocity_Out = vec3(0.0f, 1.0f, 0.0f); //genVelocity_Min;
	  color_Out = genColor;
      
	  lifetime_Out = genLifetime_Min + genLifetime_Range * randZeroOne();
	  //lifetime_Out = 3.0;
	  timeleft_Out = lifetime_Out;
      size_Out = genSize_Min + genSize_Range * randZeroOne();
	  //size_Out = 2.0;
      type_Out = 1;
      EmitVertex();
      EndPrimitive();
    }
  }
  else if(timeleft_Out > 0.0)
  {
      EmitVertex();
      EndPrimitive(); 
  }
}